/*
 * This is a basic stand-alone program to do image processing
 * with DIPlib and dipIO.
 *
 * Note that the error handling is implemented without macros. I
 * was hoping that this makes it easier to understand.
 * All functions return an error condition. If it is not DIP_OK
 * (zero), the rest of the program is skipped, the resources are
 * freed and the message is printed to stderr.
 *
 * Cris Luengo, June 2000, March 2002, September 2006
 */

/*
gcc -o standalone2 standalone2.c       \
    -I/home/diplib/dip/sunOS/include   \
    -L/home/diplib/dip/sunOS/lib       \
    -ldipio -ldip -lm
*/

#include <stdio.h> /* for fprintf */
#include <string.h> /* for strcpy */
#include "diplib.h" /* Include for basic DIPlib functions */
#include "dipio.h" /* Include to use dipIO */

#include "dip_linear.h" /* For dip_Gauss */
#include "dipio_tiff.h" /* For dipio_WriteTIFFID */

int main (void) {

  dip_Error error;
  int success = 1;
  dip_Resources rg = 0;
  dip_Image im;
  dip_String filename;
  dip_FloatArray sigmas;
  dipio_Compression compression;

  /* Initialize both DIPlib and dipIO */
  error = dip_Initialise ();
  if (error != DIP_OK) goto dip_error;
  error = dipio_Initialise ();
  if (error != DIP_OK) goto dip_error;

  /* Allocate Resources structure to keep track of allocated memory */
  error = dip_ResourcesNew (&rg, 0);
  if (error != DIP_OK) goto dip_error;

  /* This is the image we're going to use */
  error = dip_ImageNew (&im, rg);
  if (error != DIP_OK) goto dip_error;

  /* Allocate string with 1024 characters and fill with filename */
  error = dip_StringNew (&filename, 1024, "orka.tif", rg);
  if (error != DIP_OK) goto dip_error;

  /* Create 2D array with values [5,5] for the filter parameter */
  error = dip_FloatArrayNew (&sigmas, 2, 5., rg);
  if (error != DIP_OK) goto dip_error;

  /* Get the image from the file */
  error = dipio_ImageRead (im, filename, 0, DIP_FALSE, 0);
  if (error != DIP_OK) goto dip_error;

  /* Display some data on the image read */
  {
    dip_IntegerArray dimensions;
    dip_ImageType type;
    dip_DataType dataType;
    dip_int ii;

    error = dip_ImageGetDimensions (im, &dimensions, rg);
    if (error != DIP_OK) goto dip_error;
    error = dip_ImageGetType (im, &type);
    if (error != DIP_OK) goto dip_error;
    error = dip_ImageGetDataType (im, &dataType);
    if (error != DIP_OK) goto dip_error;

    printf ("\nImage read:\nNumber of dimensions: %d\n",
      dimensions->size);
    if (dimensions->size > 0) {
      printf ("Dimensions: [%d", dimensions->array[0]);
      for (ii = 1; ii < dimensions->size; ii++)
        printf (", %d", dimensions->array[ii]);
      printf ("]\n");
    }
    printf ("Image type: %s\nData type: %s\n\n",
      type == DIP_IMTP_SCALAR ? "scalar" : "unknown",
      dataType == DIP_DT_UINT8 ? "uint8" : "not uint8");
  }

  /* This is the image processing */
  error = dip_Gauss (im, im, 0, 0, sigmas, 0, -1);
  if (error != DIP_OK) goto dip_error;

  /* dip_Gauss always returns floating point types. We want uint8
     because most programs don't recognize floating-point TIFFs. */
  error = dip_ConvertDataType (im, im, DIP_DT_UINT8);
  if (error != DIP_OK) goto dip_error;

  /* The allocated space in 'filename' is large enough for this */
  strcpy (filename->string, "orka_smooth.tif");

  /* Choose a compression method and level */
  compression.method = DIPIO_CMP_DEFLATE;
  compression.level = 6;
  
  /* Write the image to another file */
  error = dipio_ImageWrite (im, filename, 0, dipio_WriteTIFFID(), compression);
  if (error != DIP_OK) goto dip_error;

dip_error:

  /* Return all allocated memory to the system */
  dip_ResourcesFree (&rg);

  /* Report errors */
  if (error) {
    /* There is an error: print the call stack. We don't go into
     * side-branches, but we could do so through lerror->next */
    dip_Error lerror = error;
    while (lerror) {
      if (lerror->message) {
        fprintf (stderr, "DIPlib Error in function %s: %s",
          lerror->function, lerror->message);
      }
      else {
        fprintf (stderr, "DIPlib Error in function %s.\n",
          lerror->function);
      }
      lerror = lerror->up;
    }
    dip_ErrorFree (error);
    success = 0;
  }

  /* Let libraries clean up after themselves */
  dipio_Exit ();
  dip_Exit ();

  return (success ? 0 : 1);
}
